#
# Definition of CARD classes for DYNA keyword input file
# Note that all method definitions in the base class, 
# with the exception of CardType(), must be overridden 
# to be of any use.
#
# The Budd Company - Technical Center
#    Charles Stuart
#    Start:  August 21, 2000
#    Last Modified:  August 22, 2000
#
#  Basic class variables include
#      - field_size - width of fields in the card, integer
#      - card - contents of card, string
#
class CARD:
  import string

  EOL = '\n'
  success = 0
  failure = 1
  cont = 2

  def __init__(self):
    self.field_size = 10              # Default spacing for fields
    self.card = '#'                   # Default card is empty comment

    """
      Allowable CARD object types are
        - comment
        - spacing_guide (special case of comment)
        - keyword
        - data

      keyword types have a variable number of data cards associated with them.
    """
    self.card_type = 'comment'        # Default CARD type

  def CardWrite(self):
    # Using the local CARD variables and format, return a string of the
    # CARD contents
    return failure

  def CardRead(self, input_string):
    # Take an input string and parse it.
    # The string should first be categorized with the CardType method,
    # so this method should never be called using the basic CARD object.
    return failure

  def CardType(self, input_string):
    # Take an input string and determine what type of CARD it represents.
    # Return an identifier for the CARD.
    #
    # First, go for the easy kills.  Is this a comment or a keyword?
    if ( input_string[0] == "$" ):
      card_type = 'comment'
      return card_type

    if ( input_string[0] == "*" ):
      card_type = 'keyword'
      return card_type

    # If we got here, it's a data card.
    card_type = 'data'
    return card_type

#
# CARD_KEYWORD Class - Inherits base functionality from the CARD class.
# The card represented by this object identifies the DYNA file as a 
# keyword input deck (as opposed to a sequence deck).
#
# This is the simplest defined object for a DYNA deck.
#
class CARD_KEYWORD(CARD):

  def __init__(self):
    self.data_cards = 0               # Number of data CARDs associated
    self.card_type = 'keyword'
    self.field_one = '*KEYWORD'

  def CardWrite(self):
    return field_one + EOL

  def CardRead(self, input_string):
    self.card = input_string
    return success

#
# CARD_TITLE Class - Inherits base functionality from the CARD class.
# The card represented by this object provides the title of the DYNA
# simulation.  One data card is associated.
#
class CARD_TITLE(CARD):

  def __init__(self):
    self.data_cards = 1               # Number of data CARDS associated
    self.card_type = 'keyword'
    self.field_one = '*TITLE'
    self.field_tuple = ( 'filename' )

  def CardWrite(self):
    self.card = self.field_one + CARD.EOL
    return self.card

  def CardRead(self, input_string):
    self.card = input_string
    return

#
# CARD_DATA Class - Inherits base functionality from the CARD class.
# This is a basic card that will most often be encapsulated within 
# another class as an add-on.  This class takes a tuple of data field
# names at instantiation, and automatically divides the 80 characters
# which make up a card into equal chunks.
#
class CARD_DATA(CARD):

  def __init__(self, field_tuple):
    self.card_type = 'data'
    self.num_fields = len(field_tuple)  # Total number of fields card may have

  def CardFillFields(self, input_string):
    return
    
  def CardWrite(self): 
    return

#
# CARD_INCLUDE Class - Inherits base functionality from the CARD class.
# The card represented by this object allows inclusion of external files in
# a DYNA deck.  One data card is associated.
#
class CARD_INCLUDE(CARD):

  def __init__(self):
    self.data_cards = 1               # Number of data CARDS associated
    self.card_type = 'keyword'
    self.field_one = '*INCLUDE'

  def CardWrite(self):
    return field_one + EOL

  def CardRead(self, input_string):
    self.card = input_string
    return
